"""Data taken from  http://www.physics.nist.gov/PhysRefData/DFTdata/"""
from __future__ import print_function, division, unicode_literals
from collections import namedtuple

# Computer generated code
_neutral = {
 'Ac': '[Rn] 6d1 7s2',
 'Ag': '[Kr] 4d10 5s1',
 'Al': '[Ne] 3s2 3p1',
 'Ar': '[Ne] 3s2 3p6',
 'As': '[Ar] 3d10 4s2 4p3',
 'At': '[Xe] 4f14 5d10 6s2 6p5',
 'Au': '[Xe] 4f14 5d10 6s1',
 'B': '[He] 2s2 2p1',
 'Ba': '[Xe] 6s2',
 'Be': '[He] 2s2',
 'Bi': '[Xe] 4f14 5d10 6s2 6p3',
 'Br': '[Ar] 3d10 4s2 4p5',
 'C': '[He] 2s2 2p2',
 'Ca': '[Ar] 4s2',
 'Cd': '[Kr] 4d10 5s2',
 'Ce': '[Xe] 4f1 5d1 6s2',
 'Cl': '[Ne] 3s2 3p5',
 'Co': '[Ar] 3d7 4s2',
 'Cr': '[Ar] 3d5 4s1',
 'Cs': '[Xe] 6s1',
 'Cu': '[Ar] 3d10 4s1',
 'Dy': '[Xe] 4f10 6s2',
 'Er': '[Xe] 4f12 6s2',
 'Eu': '[Xe] 4f7 6s2',
 'F': '[He] 2s2 2p5',
 'Fe': '[Ar] 3d6 4s2',
 'Fr': '[Rn] 7s1',
 'Ga': '[Ar] 3d10 4s2 4p1',
 'Gd': '[Xe] 4f7 5d1 6s2',
 'Ge': '[Ar] 3d10 4s2 4p2',
 'H': '1s1',
 'He': '1s2',
 'Hf': '[Xe] 4f14 5d2 6s2',
 'Hg': '[Xe] 4f14 5d10 6s2',
 'Ho': '[Xe] 4f11 6s2',
 'I': '[Kr] 4d10 5s2 5p5',
 'In': '[Kr] 4d10 5s2 5p1',
 'Ir': '[Xe] 4f14 5d7 6s2',
 'K': '[Ar] 4s1',
 'Kr': '[Ar] 3d10 4s2 4p6',
 'La': '[Xe] 5d1 6s2',
 'Li': '[He] 2s1',
 'Lu': '[Xe] 4f14 5d1 6s2',
 'Mg': '[Ne] 3s2',
 'Mn': '[Ar] 3d5 4s2',
 'Mo': '[Kr] 4d5 5s1',
 'N': '[He] 2s2 2p3',
 'Na': '[Ne] 3s1',
 'Nb': '[Kr] 4d4 5s1',
 'Nd': '[Xe] 4f4 6s2',
 'Ne': '[He] 2s2 2p6',
 'Ni': '[Ar] 3d8 4s2',
 'O': '[He] 2s2 2p4',
 'Os': '[Xe] 4f14 5d6 6s2',
 'P': '[Ne] 3s2 3p3',
 'Pa': '[Rn] 5f2 6d1 7s2',
 'Pb': '[Xe] 4f14 5d10 6s2 6p2',
 'Pd': '[Kr] 4d10',
 'Pm': '[Xe] 4f5 6s2',
 'Po': '[Xe] 4f14 5d10 6s2 6p4',
 'Pr': '[Xe] 4f3 6s2',
 'Pt': '[Xe] 4f14 5d9 6s1',
 'Ra': '[Rn] 7s2',
 'Rb': '[Kr] 5s1',
 'Re': '[Xe] 4f14 5d5 6s2',
 'Rh': '[Kr] 4d8 5s1',
 'Rn': '[Xe] 4f14 5d10 6s2 6p6',
 'Ru': '[Kr] 4d7 5s1',
 'S': '[Ne] 3s2 3p4',
 'Sb': '[Kr] 4d10 5s2 5p3',
 'Sc': '[Ar] 3d1 4s2',
 'Se': '[Ar] 3d10 4s2 4p4',
 'Si': '[Ne] 3s2 3p2',
 'Sm': '[Xe] 4f6 6s2',
 'Sn': '[Kr] 4d10 5s2 5p2',
 'Sr': '[Kr] 5s2',
 'Ta': '[Xe] 4f14 5d3 6s2',
 'Tb': '[Xe] 4f9 6s2',
 'Tc': '[Kr] 4d5 5s2',
 'Te': '[Kr] 4d10 5s2 5p4',
 'Th': '[Rn] 6d2 7s2',
 'Ti': '[Ar] 3d2 4s2',
 'Tl': '[Xe] 4f14 5d10 6s2 6p1',
 'Tm': '[Xe] 4f13 6s2',
 'U': '[Rn] 5f3 6d1 7s2',
 'V': '[Ar] 3d3 4s2',
 'W': '[Xe] 4f14 5d4 6s2',
 'Xe': '[Kr] 4d10 5s2 5p6',
 'Y': '[Kr] 4d1 5s2',
 'Yb': '[Xe] 4f14 6s2',
 'Zn': '[Ar] 3d10 4s2',
 'Zr': '[Kr] 4d2 5s2'
}

# Computer generated code
_cations = {
 'Ac': '[Rn] 7s2',
 'Ag': '[Kr] 4d10',
 'Al': '[Ne] 3s2',
 'Ar': '[Ne] 3s2 3p5',
 'As': '[Ar] 3d10 4s2 4p2',
 'At': '[Xe] 4f14 5d10 6s2 6p4',
 'Au': '[Xe] 4f14 5d10',
 'B': '[He] 2s2',
 'Ba': '[Xe] 6s1',
 'Be': '[He] 2s1',
 'Bi': '[Xe] 4f14 5d10 6s2 6p2',
 'Br': '[Ar] 3d10 4s2 4p4',
 'C': '[He] 2s2 2p1',
 'Ca': '[Ar] 4s1',
 'Cd': '[Kr] 4d10 5s1',
 'Ce': '[Xe] 4f1 5d2',
 'Cl': '[Ne] 3s2 3p4',
 'Co': '[Ar] 3d8',
 'Cr': '[Ar] 3d5',
 'Cs': '[Kr] 4d10 5s2 5p6',
 'Cu': '[Ar] 3d10',
 'Dy': '[Xe] 4f10 6s1',
 'Er': '[Xe] 4f12 6s1',
 'Eu': '[Xe] 4f7 6s1',
 'F': '[He] 2s2 2p4',
 'Fe': '[Ar] 3d6 4s1',
 'Fr': '[Xe] 4f14 5d10 6s2 6p6',
 'Ga': '[Ar] 3d10 4s2',
 'Gd': '[Xe] 4f7 5d1 6s1',
 'Ge': '[Ar] 3d10 4s2 4p1',
 'H': None,
 'He': '1s1',
 'Hf': '[Xe] 4f14 5d1 6s2',
 'Hg': '[Xe] 4f14 5d10 6s1',
 'Ho': '[Xe] 4f11 6s1',
 'I': '[Kr] 4d10 5s2 5p4',
 'In': '[Kr] 4d10 5s2',
 'Ir': '[Xe] 4f14 5d7 6s1',
 'K': '[Ne] 3s2 3p6',
 'Kr': '[Ar] 3d10 4s2 4p5',
 'La': '[Xe] 5d2',
 'Li': '1s2',
 'Lu': '[Xe] 4f14 6s2',
 'Mg': '[Ne] 3s1',
 'Mn': '[Ar] 3d5 4s1',
 'Mo': '[Kr] 4d5',
 'N': '[He] 2s2 2p2',
 'Na': '[He] 2s2 2p6',
 'Nb': '[Kr] 4d4',
 'Nd': '[Xe] 4f4 6s1',
 'Ne': '[He] 2s2 2p5',
 'Ni': '[Ar] 3d9',
 'O': '[He] 2s2 2p3',
 'Os': '[Xe] 4f14 5d6 6s1',
 'P': '[Ne] 3s2 3p2',
 'Pa': '[Rn] 5f2 7s2',
 'Pb': '[Xe] 4f14 5d10 6s2 6p1',
 'Pd': '[Kr] 4d9',
 'Pm': '[Xe] 4f5 6s1',
 'Po': '[Xe] 4f14 5d10 6s2 6p3',
 'Pr': '[Xe] 4f3 6s1',
 'Pt': '[Xe] 4f14 5d9',
 'Ra': '[Rn] 7s1',
 'Rb': '[Ar] 3d10 4s2 4p6',
 'Re': '[Xe] 4f14 5d5 6s1',
 'Rh': '[Kr] 4d8',
 'Rn': '[Xe] 4f14 5d10 6s2 6p5',
 'Ru': '[Kr] 4d7',
 'S': '[Ne] 3s2 3p3',
 'Sb': '[Kr] 4d10 5s2 5p2',
 'Sc': '[Ar] 3d1 4s1',
 'Se': '[Ar] 3d10 4s2 4p3',
 'Si': '[Ne] 3s2 3p1',
 'Sm': '[Xe] 4f6 6s1',
 'Sn': '[Kr] 4d10 5s2 5p1',
 'Sr': '[Kr] 5s1',
 'Ta': '[Xe] 4f14 5d3 6s1',
 'Tb': '[Xe] 4f9 6s1',
 'Tc': '[Kr] 4d5 5s1',
 'Te': '[Kr] 4d10 5s2 5p3',
 'Th': '[Rn] 6d2 7s1',
 'Ti': '[Ar] 3d2 4s1',
 'Tl': '[Xe] 4f14 5d10 6s2',
 'Tm': '[Xe] 4f13 6s1',
 'U': '[Rn] 5f3 7s2',
 'V': '[Ar] 3d4',
 'W': '[Xe] 4f14 5d4 6s1',
 'Xe': '[Kr] 4d10 5s2 5p5',
 'Y': '[Kr] 5s2',
 'Yb': '[Xe] 4f14 6s1',
 'Zn': '[Ar] 3d10 4s1',
 'Zr': '[Kr] 4d2 5s1'
}

# Computer generated code: nist_xctype = LDA, iontype = neutrals
# format:
#    'element': (atomic number, [(n, l, occ, energy), ...])
_neutral_entries = {
 'Ac': (89,
        [(1, 0, 2, -3443.110367),
         (2, 0, 2, -592.622878),
         (2, 1, 6, -572.7627),
         (3, 0, 2, -147.320716),
         (3, 1, 6, -137.654394),
         (3, 2, 10, -119.541743),
         (4, 0, 2, -36.15826),
         (4, 1, 6, -31.761846),
         (4, 2, 10, -23.57061),
         (5, 0, 2, -7.713078),
         (5, 1, 6, -6.06511),
         (4, 3, 14, -12.278225),
         (5, 2, 10, -3.222752),
         (6, 0, 2, -1.19698),
         (6, 1, 6, -0.744524),
         (6, 2, 1, -0.137786),
         (7, 0, 2, -0.126551)],
        {'Ecoul': 9262.611883,
         'Eenuc': -56286.016507,
         'Ekin': 23709.594347,
         'Etot': -23716.496952,
         'Exc': -402.686674}),
 'Ag': (47,
        [(1, 0, 2, -900.324578),
         (2, 0, 2, -129.859807),
         (2, 1, 6, -120.913351),
         (3, 0, 2, -23.678437),
         (3, 1, 6, -20.06763),
         (3, 2, 10, -13.367803),
         (4, 0, 2, -3.22309),
         (4, 1, 6, -2.086602),
         (4, 2, 10, -0.298706),
         (5, 0, 1, -0.157407)],
        {'Ecoul': 2131.618463,
         'Eenuc': -12377.294484,
         'Ekin': 5191.8186,
         'Etot': -5195.031215,
         'Exc': -141.173794}),
 'Al': (13,
        [(1, 0, 2, -55.156044),
         (2, 0, 2, -3.934827),
         (2, 1, 6, -2.564018),
         (3, 0, 2, -0.286883),
         (3, 1, 1, -0.102545)],
        {'Ecoul': 112.670733,
         'Eenuc': -577.205757,
         'Ekin': 240.663489,
         'Etot': -241.315573,
         'Exc': -17.444038}),
 'Ar': (18,
        [(1, 0, 2, -113.800134),
         (2, 0, 2, -10.794172),
         (2, 1, 6, -8.443439),
         (3, 0, 2, -0.883384),
         (3, 1, 6, -0.38233)],
        {'Ecoul': 231.458124,
         'Eenuc': -1253.131982,
         'Ekin': 524.969812,
         'Etot': -525.946195,
         'Exc': -29.242149}),
 'As': (33,
        [(1, 0, 2, -423.336658),
         (2, 0, 2, -53.093086),
         (2, 1, 6, -47.527869),
         (3, 0, 2, -6.730755),
         (3, 1, 6, -4.851725),
         (3, 2, 10, -1.542767),
         (4, 0, 2, -0.52367),
         (4, 1, 3, -0.197497)],
        {'Ecoul': 960.716547,
         'Eenuc': -5344.04275,
         'Ekin': 2230.434545,
         'Etot': -2232.534978,
         'Exc': -79.643321}),
 'At': (85,
        [(1, 0, 2, -3127.390276),
         (2, 0, 2, -531.81835),
         (2, 1, 6, -513.044243),
         (3, 0, 2, -129.035542),
         (3, 1, 6, -119.995013),
         (3, 2, 10, -103.060375),
         (4, 0, 2, -29.809515),
         (4, 1, 6, -25.778264),
         (4, 2, 10, -18.295162),
         (5, 0, 2, -5.453383),
         (5, 1, 6, -4.027061),
         (4, 3, 14, -8.063483),
         (5, 2, 10, -1.643758),
         (6, 0, 2, -0.560189),
         (6, 1, 5, -0.255453)],
        {'Ecoul': 8414.956548,
         'Eenuc': -50556.606342,
         'Ekin': 21254.968349,
         'Etot': -21261.555215,
         'Exc': -374.873771}),
 'Au': (79,
        [(1, 0, 2, -2683.508245),
         (2, 0, 2, -447.888973),
         (2, 1, 6, -430.725701),
         (3, 0, 2, -104.824516),
         (3, 1, 6, -96.707),
         (3, 2, 10, -81.511751),
         (4, 0, 2, -22.078357),
         (4, 1, 6, -18.578652),
         (4, 2, 10, -12.131815),
         (5, 0, 2, -3.113936),
         (5, 1, 6, -2.002495),
         (4, 3, 14, -3.486824),
         (5, 2, 10, -0.304738),
         (6, 0, 1, -0.162334)],
        {'Ecoul': 7170.546095,
         'Eenuc': -42552.587217,
         'Ekin': 17854.711365,
         'Etot': -17860.790943,
         'Exc': -333.461185}),
 'B': (5,
       [(1, 0, 2, -6.564347), (2, 0, 2, -0.344701), (2, 1, 1, -0.136603)],
       {'Ecoul': 11.503002,
        'Eenuc': -56.484587,
        'Ekin': 24.161047,
        'Etot': -24.344198,
        'Exc': -3.52366}),
 'Ba': (56,
        [(1, 0, 2, -1305.743258),
         (2, 0, 2, -200.844444),
         (2, 1, 6, -189.598483),
         (3, 0, 2, -42.359434),
         (3, 1, 6, -37.536931),
         (3, 2, 10, -28.528933),
         (4, 0, 2, -8.257061),
         (4, 1, 6, -6.497622),
         (4, 2, 10, -3.432441),
         (5, 0, 2, -1.157159),
         (5, 1, 6, -0.698605),
         (6, 0, 2, -0.118967)],
        {'Ecoul': 3098.114585,
         'Eenuc': -18668.853113,
         'Ekin': 7876.220134,
         'Etot': -7880.111578,
         'Exc': -185.593184}),
 'Be': (4,
        [(1, 0, 2, -3.856411), (2, 0, 2, -0.205744)],
        {'Ecoul': 7.115257,
         'Eenuc': -33.357034,
         'Ekin': 14.309424,
         'Etot': -14.447209,
         'Exc': -2.514856}),
 'Bi': (83,
        [(1, 0, 2, -2975.550959),
         (2, 0, 2, -502.950758),
         (2, 1, 6, -484.716359),
         (3, 0, 2, -120.613998),
         (3, 1, 6, -111.883393),
         (3, 2, 10, -95.532476),
         (4, 0, 2, -27.07034),
         (4, 1, 6, -23.218641),
         (4, 2, 10, -16.084817),
         (5, 0, 2, -4.611934),
         (5, 1, 6, -3.293637),
         (4, 3, 14, -6.382744),
         (5, 2, 10, -1.139408),
         (6, 0, 2, -0.426129),
         (6, 1, 3, -0.180198)],
        {'Ecoul': 7989.999458,
         'Eenuc': -47803.484105,
         'Ekin': 20083.999522,
         'Etot': -20090.414449,
         'Exc': -360.929324}),
 'Br': (35,
        [(1, 0, 2, -480.182643),
         (2, 0, 2, -61.710022),
         (2, 1, 6, -55.67796),
         (3, 0, 2, -8.409057),
         (3, 1, 6, -6.298805),
         (3, 2, 10, -2.52211),
         (4, 0, 2, -0.720066),
         (4, 1, 5, -0.295334)],
        {'Ecoul': 1098.492305,
         'Eenuc': -6149.802515,
         'Ekin': 2568.365427,
         'Etot': -2570.6207,
         'Exc': -87.675917}),
 'C': (6,
       [(1, 0, 2, -9.947718), (2, 0, 2, -0.500866), (2, 1, 2, -0.199186)],
       {'Ecoul': 17.627997,
        'Eenuc': -87.515412,
        'Ekin': 37.190391,
        'Etot': -37.425749,
        'Exc': -4.728724}),
 'Ca': (20,
        [(1, 0, 2, -143.935181),
         (2, 0, 2, -15.046905),
         (2, 1, 6, -12.285376),
         (3, 0, 2, -1.706331),
         (3, 1, 6, -1.030572),
         (4, 0, 2, -0.141411)],
        {'Ecoul': 285.20613,
         'Eenuc': -1601.463209,
         'Ekin': 674.657334,
         'Etot': -675.742283,
         'Exc': -34.142538}),
 'Cd': (48,
        [(1, 0, 2, -941.476646),
         (2, 0, 2, -136.83249),
         (2, 1, 6, -127.63512),
         (3, 0, 2, -25.379908),
         (3, 1, 6, -21.637522),
         (3, 2, 10, -14.685252),
         (4, 0, 2, -3.596069),
         (4, 1, 6, -2.39526),
         (4, 2, 10, -0.47053),
         (5, 0, 2, -0.204228)],
        {'Ecoul': 2233.419308,
         'Eenuc': -13008.902423,
         'Ekin': 5459.099922,
         'Etot': -5462.390982,
         'Exc': -146.007788}),
 'Ce': (58,
        [(1, 0, 2, -1406.148284),
         (2, 0, 2, -218.684842),
         (2, 1, 6, -206.925148),
         (3, 0, 2, -47.035283),
         (3, 1, 6, -41.938282),
         (3, 2, 10, -32.412569),
         (4, 0, 2, -9.432744),
         (4, 1, 6, -7.532106),
         (4, 2, 10, -4.192548),
         (5, 0, 2, -1.369728),
         (5, 1, 6, -0.85011),
         (4, 3, 1, -0.337442),
         (5, 2, 1, -0.14055),
         (6, 0, 2, -0.133974)],
        {'Ecoul': 3351.833535,
         'Eenuc': -20278.178413,
         'Ekin': 8559.301423,
         'Etot': -8563.360285,
         'Exc': -196.316831}),
 'Cl': (17,
        [(1, 0, 2, -100.369229),
         (2, 0, 2, -9.187993),
         (2, 1, 6, -7.039982),
         (3, 0, 2, -0.754458),
         (3, 1, 5, -0.32038)],
        {'Ecoul': 202.772337,
         'Eenuc': -1092.568993,
         'Ekin': 457.758293,
         'Etot': -458.664179,
         'Exc': -26.625816}),
 'Co': (27,
        [(1, 0, 2, -275.616639),
         (2, 0, 2, -32.379758),
         (2, 1, 6, -28.152095),
         (3, 0, 2, -3.651812),
         (3, 1, 6, -2.388285),
         (3, 2, 7, -0.322368),
         (4, 0, 2, -0.204497)],
        {'Ecoul': 589.169216,
         'Eenuc': -3291.260194,
         'Ekin': 1378.468819,
         'Etot': -1380.091264,
         'Exc': -56.469106}),
 'Cr': (24,
        [(1, 0, 2, -213.881191),
         (2, 0, 2, -24.113457),
         (2, 1, 6, -20.526273),
         (3, 0, 2, -2.649085),
         (3, 1, 6, -1.65423),
         (3, 2, 5, -0.118123),
         (4, 0, 1, -0.150445)],
        {'Ecoul': 441.867987,
         'Eenuc': -2478.305541,
         'Ekin': 1040.642716,
         'Etot': -1042.030238,
         'Exc': -46.2354}),
 'Cs': (55,
        [(1, 0, 2, -1256.738791),
         (2, 0, 2, -191.981873),
         (2, 1, 6, -180.995344),
         (3, 0, 2, -39.851584),
         (3, 1, 6, -35.166423),
         (3, 2, 10, -26.418398),
         (4, 0, 2, -7.455966),
         (4, 1, 6, -5.769326),
         (4, 2, 10, -2.848386),
         (5, 0, 2, -0.915819),
         (5, 1, 6, -0.504903),
         (6, 0, 1, -0.078699)],
        {'Ecoul': 2988.242134,
         'Eenuc': -17904.90606,
         'Ekin': 7546.734488,
         'Etot': -7550.55771,
         'Exc': -180.628271}),
 'Cu': (29,
        [(1, 0, 2, -320.78852),
         (2, 0, 2, -38.14131),
         (2, 1, 6, -33.481247),
         (3, 0, 2, -4.057453),
         (3, 1, 6, -2.609244),
         (3, 2, 10, -0.202272),
         (4, 0, 1, -0.172056)],
        {'Ecoul': 713.928477,
         'Eenuc': -3923.329381,
         'Ekin': 1635.9828,
         'Etot': -1637.785861,
         'Exc': -64.367756}),
 'Dy': (66,
        [(1, 0, 2, -1843.229585),
         (2, 0, 2, -295.342856),
         (2, 1, 6, -281.558531),
         (3, 0, 2, -65.299442),
         (3, 1, 6, -59.091931),
         (3, 2, 10, -47.4867),
         (4, 0, 2, -12.551251),
         (4, 1, 6, -10.094091),
         (4, 2, 10, -5.686352),
         (5, 0, 2, -1.547977),
         (5, 1, 6, -0.90349),
         (4, 3, 10, -0.265302),
         (6, 0, 2, -0.132769)],
        {'Ecoul': 4637.607532,
         'Eenuc': -27663.021946,
         'Ekin': 11633.033352,
         'Etot': -11637.869664,
         'Exc': -245.488603}),
 'Er': (68,
        [(1, 0, 2, -1961.799176),
         (2, 0, 2, -316.310631),
         (2, 1, 6, -302.01827),
         (3, 0, 2, -70.310142),
         (3, 1, 6, -63.818655),
         (3, 2, 10, -51.682149),
         (4, 0, 2, -13.423547),
         (4, 1, 6, -10.819574),
         (4, 2, 10, -6.127443),
         (5, 0, 2, -1.616073),
         (5, 1, 6, -0.935202),
         (4, 3, 12, -0.278577),
         (6, 0, 2, -0.134905)],
        {'Ecoul': 5005.110005,
         'Eenuc': -29730.765707,
         'Ekin': 12489.682402,
         'Etot': -12494.718304,
         'Exc': -258.745003}),
 'Eu': (63,
        [(1, 0, 2, -1672.309322),
         (2, 0, 2, -265.199534),
         (2, 1, 6, -252.176697),
         (3, 0, 2, -58.068128),
         (3, 1, 6, -52.281987),
         (3, 2, 10, -41.465518),
         (4, 0, 2, -11.267747),
         (4, 1, 6, -9.025455),
         (4, 2, 10, -5.03242),
         (5, 0, 2, -1.444087),
         (5, 1, 6, -0.853575),
         (4, 3, 7, -0.232773),
         (6, 0, 2, -0.129426)],
        {'Ecoul': 4124.189856,
         'Eenuc': -24732.67422,
         'Ekin': 10415.168162,
         'Etot': -10419.710775,
         'Exc': -226.394574}),
 'F': (9,
       [(1, 0, 2, -24.189391), (2, 0, 2, -1.086859), (2, 1, 5, -0.415606)],
       {'Ecoul': 49.535385,
        'Eenuc': -237.688846,
        'Ekin': 98.676131,
        'Etot': -99.099648,
        'Exc': -9.622319}),
 'Fe': (26,
        [(1, 0, 2, -254.225505),
         (2, 0, 2, -29.56486),
         (2, 1, 6, -25.551766),
         (3, 0, 2, -3.360621),
         (3, 1, 6, -2.187523),
         (3, 2, 6, -0.295049),
         (4, 0, 2, -0.197978)],
        {'Ecoul': 535.295832,
         'Eenuc': -3003.082484,
         'Ekin': 1259.553429,
         'Etot': -1261.093056,
         'Exc': -52.859833}),
 'Fr': (87,
        [(1, 0, 2, -3283.263399),
         (2, 0, 2, -561.73045),
         (2, 1, 6, -542.41424),
         (3, 0, 2, -137.959632),
         (3, 1, 6, -128.607136),
         (3, 2, 10, -111.085223),
         (4, 0, 2, -32.861013),
         (4, 1, 6, -28.648131),
         (4, 2, 10, -20.812462),
         (5, 0, 2, -6.509516),
         (5, 1, 6, -4.97328),
         (4, 3, 14, -10.050648),
         (5, 2, 10, -2.360991),
         (6, 0, 2, -0.841848),
         (6, 1, 6, -0.466197),
         (7, 0, 1, -0.076176)],
        {'Ecoul': 8837.303934,
         'Eenuc': -53382.425806,
         'Ekin': 22463.57381,
         'Etot': -22470.319655,
         'Exc': -388.771593}),
 'Ga': (31,
        [(1, 0, 2, -370.170639),
         (2, 0, 2, -45.200869),
         (2, 1, 6, -40.093339),
         (3, 0, 2, -5.241645),
         (3, 1, 6, -3.584666),
         (3, 2, 10, -0.736204),
         (4, 0, 2, -0.328019),
         (4, 1, 1, -0.101634)],
        {'Ecoul': 833.754978,
         'Eenuc': -4603.576824,
         'Ekin': 1919.894783,
         'Etot': -1921.846456,
         'Exc': -71.919393}),
 'Gd': (64,
        [(1, 0, 2, -1728.625195),
         (2, 0, 2, -275.36313),
         (2, 1, 6, -262.081616),
         (3, 0, 2, -60.764408),
         (3, 1, 6, -54.836922),
         (3, 2, 10, -43.754556),
         (4, 0, 2, -11.986486),
         (4, 1, 6, -9.669866),
         (4, 2, 10, -5.531835),
         (5, 0, 2, -1.608477),
         (5, 1, 6, -0.978749),
         (4, 3, 7, -0.489012),
         (5, 2, 1, -0.12722),
         (6, 0, 2, -0.143627)],
        {'Ecoul': 4273.725204,
         'Eenuc': -25670.165341,
         'Ekin': 10812.02673,
         'Etot': -10816.653877,
         'Exc': -232.24047}),
 'Ge': (32,
        [(1, 0, 2, -396.292991),
         (2, 0, 2, -49.055282),
         (2, 1, 6, -43.720129),
         (3, 0, 2, -5.961472),
         (3, 1, 6, -4.194822),
         (3, 2, 10, -1.117316),
         (4, 0, 2, -0.426523),
         (4, 1, 2, -0.149882)],
        {'Ecoul': 895.945638,
         'Eenuc': -4965.789424,
         'Ekin': 2071.782093,
         'Etot': -2073.807332,
         'Exc': -75.74564}),
 'H': (1,
       [(1, 0, 1, -0.233471)],
       {'Ecoul': 0.282827,
        'Eenuc': -0.920999,
        'Ekin': 0.425027,
        'Etot': -0.445671,
        'Exc': -0.232525}),
 'He': (2,
        [(1, 0, 2, -0.570425)],
        {'Ecoul': 1.99612,
         'Eenuc': -6.625564,
         'Ekin': 2.767922,
         'Etot': -2.834836,
         'Exc': -0.973314}),
 'Hf': (72,
        [(1, 0, 2, -2210.65199),
         (2, 0, 2, -361.006527),
         (2, 1, 6, -345.687023),
         (3, 0, 2, -81.522812),
         (3, 1, 6, -74.452656),
         (3, 2, 10, -61.231443),
         (4, 0, 2, -15.883625),
         (4, 1, 6, -12.971211),
         (4, 2, 10, -7.676638),
         (5, 0, 2, -2.049828),
         (5, 1, 6, -1.246441),
         (4, 3, 14, -0.871574),
         (5, 2, 2, -0.143805),
         (6, 0, 2, -0.166465)],
        {'Ecoul': 5760.587419,
         'Eenuc': -34104.629307,
         'Ekin': 14312.076902,
         'Etot': -14317.49372,
         'Exc': -285.528734}),
 'Hg': (80,
        [(1, 0, 2, -2755.022637),
         (2, 0, 2, -461.27864),
         (2, 1, 6, -443.848676),
         (3, 0, 2, -108.597921),
         (3, 1, 6, -100.328031),
         (3, 2, 10, -84.845492),
         (4, 0, 2, -23.222921),
         (4, 1, 6, -19.636187),
         (4, 2, 10, -13.019221),
         (5, 0, 2, -3.423486),
         (5, 1, 6, -2.261975),
         (4, 3, 14, -4.110291),
         (5, 2, 10, -0.452552),
         (6, 0, 2, -0.205137)],
        {'Ecoul': 7374.994996,
         'Eenuc': -43837.047179,
         'Ekin': 18398.108387,
         'Etot': -18404.27422,
         'Exc': -340.330424}),
 'Ho': (67,
        [(1, 0, 2, -1902.051908),
         (2, 0, 2, -305.739294),
         (2, 1, 6, -291.700994),
         (3, 0, 2, -67.785492),
         (3, 1, 6, -61.436304),
         (3, 2, 10, -49.565996),
         (4, 0, 2, -12.985498),
         (4, 1, 6, -10.455303),
         (4, 2, 10, -5.906195),
         (5, 0, 2, -1.582088),
         (5, 1, 6, -0.919463),
         (4, 3, 11, -0.272677),
         (6, 0, 2, -0.133845)],
        {'Ecoul': 4818.79662,
         'Eenuc': -28685.338027,
         'Ekin': 12056.834803,
         'Etot': -12061.770549,
         'Exc': -252.063945}),
 'I': (53,
       [(1, 0, 2, -1161.787047),
        (2, 0, 2, -175.073804),
        (2, 1, 6, -164.603788),
        (3, 0, 2, -35.243351),
        (3, 1, 6, -30.831092),
        (3, 2, 10, -22.600693),
        (4, 0, 2, -6.115811),
        (4, 1, 6, -4.572522),
        (4, 2, 10, -1.938179),
        (5, 0, 2, -0.596339),
        (5, 1, 5, -0.267904)],
       {'Ecoul': 2766.012734,
        'Eenuc': -16421.269549,
        'Ekin': 6911.09591,
        'Etot': -6914.773092,
        'Exc': -170.612187}),
 'In': (49,
        [(1, 0, 2, -983.647445),
         (2, 0, 2, -144.078357),
         (2, 1, 6, -134.628845),
         (3, 0, 2, -27.2206),
         (3, 1, 6, -23.345778),
         (3, 2, 10, -16.139823),
         (4, 0, 2, -4.062639),
         (4, 1, 6, -2.795832),
         (4, 2, 10, -0.730481),
         (5, 0, 2, -0.290497),
         (5, 1, 1, -0.101782)],
        {'Ecoul': 2334.432447,
         'Eenuc': -13654.870647,
         'Ekin': 5733.94412,
         'Etot': -5737.309064,
         'Exc': -150.814984}),
 'Ir': (77,
        [(1, 0, 2, -2543.761342),
         (2, 0, 2, -422.159424),
         (2, 1, 6, -405.526834),
         (3, 0, 2, -97.923081),
         (3, 1, 6, -90.108427),
         (3, 2, 10, -75.485027),
         (4, 0, 2, -20.29429),
         (4, 1, 6, -16.966578),
         (4, 2, 10, -10.856593),
         (5, 0, 2, -2.909174),
         (5, 1, 6, -1.883349),
         (4, 3, 14, -2.738339),
         (5, 2, 7, -0.335189),
         (6, 0, 2, -0.195511)],
        {'Ecoul': 6740.648988,
         'Eenuc': -40018.842379,
         'Ekin': 16795.798049,
         'Etot': -16801.677471,
         'Exc': -319.282128}),
 'K': (19,
       [(1, 0, 2, -128.414957),
        (2, 0, 2, -12.839001),
        (2, 1, 6, -10.283851),
        (3, 0, 2, -1.281897),
        (3, 1, 6, -0.693776),
        (4, 0, 1, -0.088815)],
       {'Ecoul': 257.381524,
        'Eenuc': -1421.104115,
        'Ekin': 597.17213,
        'Etot': -598.20059,
        'Exc': -31.650129}),
 'Kr': (36,
        [(1, 0, 2, -509.982989),
         (2, 0, 2, -66.285953),
         (2, 1, 6, -60.017328),
         (3, 0, 2, -9.315192),
         (3, 1, 6, -7.086634),
         (3, 2, 10, -3.074109),
         (4, 0, 2, -0.820574),
         (4, 1, 6, -0.34634)],
        {'Ecoul': 1171.723688,
         'Eenuc': -6577.865761,
         'Ekin': 2747.813142,
         'Etot': -2750.14794,
         'Exc': -91.819009}),
 'La': (57,
        [(1, 0, 2, -1355.622446),
         (2, 0, 2, -209.831151),
         (2, 1, 6, -198.325243),
         (3, 0, 2, -44.856283),
         (3, 1, 6, -39.895838),
         (3, 2, 10, -30.626696),
         (4, 0, 2, -9.000543),
         (4, 1, 6, -7.167724),
         (4, 2, 10, -3.95801),
         (5, 0, 2, -1.324936),
         (5, 1, 6, -0.824498),
         (5, 2, 1, -0.141085),
         (6, 0, 2, -0.132233)],
        {'Ecoul': 3215.018109,
         'Eenuc': -19455.494762,
         'Ekin': 8213.607378,
         'Etot': -8217.57523,
         'Exc': -190.705955}),
 'Li': (3,
        [(1, 0, 2, -1.878564), (2, 0, 1, -0.10554)],
        {'Ecoul': 3.990755,
         'Eenuc': -16.911041,
         'Ekin': 7.237484,
         'Etot': -7.335195,
         'Exc': -1.652393}),
 'Lu': (71,
        [(1, 0, 2, -2146.885351),
         (2, 0, 2, -349.390492),
         (2, 1, 6, -334.330902),
         (3, 0, 2, -78.462398),
         (3, 1, 6, -71.538779),
         (3, 2, 10, -58.592982),
         (4, 0, 2, -15.08337),
         (4, 1, 6, -12.250904),
         (4, 2, 10, -7.113364),
         (5, 0, 2, -1.872086),
         (5, 1, 6, -1.111991),
         (4, 3, 14, -0.568096),
         (5, 2, 1, -0.103686),
         (6, 0, 2, -0.155112)],
        {'Ecoul': 5575.225315,
         'Eenuc': -32987.406799,
         'Ekin': 13842.903527,
         'Etot': -13848.230375,
         'Exc': -278.952417}),
 'Mg': (12,
        [(1, 0, 2, -45.973167),
         (2, 0, 2, -2.903746),
         (2, 1, 6, -1.71897),
         (3, 0, 2, -0.175427)],
        {'Ecoul': 95.67329,
         'Eenuc': -477.899149,
         'Ekin': 198.541505,
         'Etot': -199.139406,
         'Exc': -15.455051}),
 'Mn': (25,
        [(1, 0, 2, -233.696912),
         (2, 0, 2, -26.866646),
         (2, 1, 6, -23.066297),
         (3, 0, 2, -3.076637),
         (3, 1, 6, -1.99145),
         (3, 2, 5, -0.26654),
         (4, 0, 2, -0.191136)],
        {'Ecoul': 485.085158,
         'Eenuc': -2731.133992,
         'Ekin': 1146.990867,
         'Etot': -1148.449372,
         'Exc': -49.391406}),
 'Mo': (42,
        [(1, 0, 2, -709.232119),
         (2, 0, 2, -98.503638),
         (2, 1, 6, -90.791541),
         (3, 0, 2, -16.681545),
         (3, 1, 6, -13.71481),
         (3, 2, 10, -8.257721),
         (4, 0, 2, -2.234824),
         (4, 1, 6, -1.39005),
         (4, 2, 5, -0.153347),
         (5, 0, 1, -0.14788)],
        {'Ecoul': 1638.977454,
         'Eenuc': -9465.024542,
         'Ekin': 3970.231601,
         'Etot': -3973.013235,
         'Exc': -117.197748}),
 'N': (7,
       [(1, 0, 2, -14.011501), (2, 0, 2, -0.676151), (2, 1, 3, -0.266297)],
       {'Ecoul': 25.799845,
        'Eenuc': -127.416229,
        'Ekin': 53.731727,
        'Etot': -54.025016,
        'Exc': -6.140359}),
 'Na': (11,
        [(1, 0, 2, -37.719975),
         (2, 0, 2, -2.063401),
         (2, 1, 6, -1.060636),
         (3, 0, 1, -0.103415)],
        {'Ecoul': 79.732497,
         'Eenuc': -388.546372,
         'Ekin': 160.896571,
         'Etot': -161.44006,
         'Exc': -13.522757}),
 'Nb': (41,
        [(1, 0, 2, -673.76253),
         (2, 0, 2, -92.74086),
         (2, 1, 6, -85.272175),
         (3, 0, 2, -15.393727),
         (3, 1, 6, -12.552855),
         (3, 2, 10, -7.339839),
         (4, 0, 2, -2.036693),
         (4, 1, 6, -1.250049),
         (4, 2, 4, -0.125252),
         (5, 0, 1, -0.144272)],
        {'Ecoul': 1551.501122,
         'Eenuc': -8938.494135,
         'Ekin': 3748.49657,
         'Etot': -3751.196175,
         'Exc': -112.699732}),
 'Nd': (60,
        [(1, 0, 2, -1509.698955),
         (2, 0, 2, -236.613572),
         (2, 1, 6, -224.351816),
         (3, 0, 2, -51.161263),
         (3, 1, 6, -45.791219),
         (3, 2, 10, -35.754515),
         (4, 0, 2, -10.000891),
         (4, 1, 6, -7.96782),
         (4, 2, 10, -4.377027),
         (5, 0, 2, -1.334934),
         (5, 1, 6, -0.798503),
         (4, 3, 4, -0.179508),
         (6, 0, 2, -0.125796)],
        {'Ecoul': 3654.948015,
         'Eenuc': -22003.064698,
         'Ekin': 9276.05444,
         'Etot': -9280.311037,
         'Exc': -208.248794}),
 'Ne': (10,
        [(1, 0, 2, -30.305855), (2, 0, 2, -1.322809), (2, 1, 6, -0.498034)],
        {'Ecoul': 65.726488,
         'Eenuc': -309.988206,
         'Ekin': 127.738667,
         'Etot': -128.233481,
         'Exc': -11.71043}),
 'Ni': (28,
        [(1, 0, 2, -297.870824),
         (2, 0, 2, -35.312112),
         (2, 1, 6, -30.868027),
         (3, 0, 2, -3.950717),
         (3, 1, 6, -2.594158),
         (3, 2, 8, -0.348699),
         (4, 0, 2, -0.210764)],
        {'Ecoul': 646.833302,
         'Eenuc': -3596.066045,
         'Ekin': 1503.873332,
         'Etot': -1505.580197,
         'Exc': -60.220786}),
 'O': (8,
       [(1, 0, 2, -18.758245), (2, 0, 2, -0.871362), (2, 1, 4, -0.338381)],
       {'Ecoul': 36.331102,
        'Eenuc': -177.152578,
        'Ekin': 74.116881,
        'Etot': -74.473077,
        'Exc': -7.768482}),
 'Os': (76,
        [(1, 0, 2, -2475.238617),
         (2, 0, 2, -409.522396),
         (2, 1, 6, -393.15408),
         (3, 0, 2, -94.501324),
         (3, 1, 6, -86.837047),
         (3, 2, 10, -72.497183),
         (4, 0, 2, -19.362527),
         (4, 1, 6, -16.119671),
         (4, 2, 10, -10.176082),
         (5, 0, 2, -2.738293),
         (5, 1, 6, -1.757404),
         (4, 3, 14, -2.321175),
         (5, 2, 6, -0.296791),
         (6, 0, 2, -0.191489)],
        {'Ecoul': 6537.173416,
         'Eenuc': -38791.574393,
         'Ekin': 16280.510034,
         'Etot': -16286.295408,
         'Exc': -312.404466}),
 'P': (15,
       [(1, 0, 2, -76.061897),
        (2, 0, 2, -6.329346),
        (2, 1, 6, -4.576617),
        (3, 0, 2, -0.512364),
        (3, 1, 3, -0.20608)],
       {'Ecoul': 153.065007,
        'Eenuc': -810.4001,
        'Ekin': 339.173459,
        'Etot': -339.946219,
        'Exc': -21.784585}),
 'Pa': (91,
        [(1, 0, 2, -3606.333629),
         (2, 0, 2, -623.870431),
         (2, 1, 6, -603.470278),
         (3, 0, 2, -156.466742),
         (3, 1, 6, -146.485678),
         (3, 2, 10, -127.781168),
         (4, 0, 2, -39.064507),
         (4, 1, 6, -34.48293),
         (4, 2, 10, -25.933121),
         (5, 0, 2, -8.463463),
         (5, 1, 6, -6.709821),
         (4, 3, 14, -14.105747),
         (5, 2, 10, -3.659928),
         (6, 0, 2, -1.287232),
         (6, 1, 6, -0.799756),
         (5, 3, 2, -0.316813),
         (6, 2, 1, -0.142481),
         (7, 0, 2, -0.129653)],
        {'Ecoul': 9743.171707,
         'Eenuc': -59321.169428,
         'Ekin': 24994.200598,
         'Etot': -25001.291382,
         'Exc': -417.494259}),
 'Pb': (82,
        [(1, 0, 2, -2901.078061),
         (2, 0, 2, -488.843335),
         (2, 1, 6, -470.877785),
         (3, 0, 2, -116.526852),
         (3, 1, 6, -107.950391),
         (3, 2, 10, -91.889924),
         (4, 0, 2, -25.75333),
         (4, 1, 6, -21.990564),
         (4, 2, 10, -15.030026),
         (5, 0, 2, -4.206797),
         (5, 1, 6, -2.941657),
         (4, 3, 14, -5.592532),
         (5, 2, 10, -0.902393),
         (6, 0, 2, -0.357187),
         (6, 1, 2, -0.141831)],
        {'Ecoul': 7781.994459,
         'Eenuc': -46459.627742,
         'Ekin': 19512.662684,
         'Etot': -19518.993145,
         'Exc': -354.022546}),
 'Pd': (46,
        [(1, 0, 2, -860.134909),
         (2, 0, 2, -123.105078),
         (2, 1, 6, -114.408286),
         (3, 0, 2, -22.060898),
         (3, 1, 6, -18.580798),
         (3, 2, 10, -12.132197),
         (4, 0, 2, -2.889173),
         (4, 1, 6, -1.815215),
         (4, 2, 10, -0.160771)],
        {'Ecoul': 2031.645838,
         'Eenuc': -11762.875805,
         'Ekin': 4932.234516,
         'Etot': -4935.368406,
         'Exc': -136.372955}),
 'Pm': (61,
        [(1, 0, 2, -1562.980284),
         (2, 0, 2, -245.970548),
         (2, 1, 6, -233.455114),
         (3, 0, 2, -53.429311),
         (3, 1, 6, -47.921132),
         (3, 2, 10, -37.625433),
         (4, 0, 2, -10.422756),
         (4, 1, 6, -8.320495),
         (4, 2, 10, -4.596822),
         (5, 0, 2, -1.372265),
         (5, 1, 6, -0.817702),
         (4, 3, 5, -0.200159),
         (6, 0, 2, -0.127053)],
        {'Ecoul': 3806.516511,
         'Eenuc': -22890.942295,
         'Ekin': 9647.133137,
         'Etot': -9651.484134,
         'Exc': -214.191488}),
 'Po': (84,
        [(1, 0, 2, -3050.988417),
         (2, 0, 2, -517.275843),
         (2, 1, 6, -498.77192),
         (3, 0, 2, -124.783683),
         (3, 1, 6, -115.898384),
         (3, 2, 10, -99.256068),
         (4, 0, 2, -28.42254),
         (4, 1, 6, -24.481337),
         (4, 2, 10, -17.173307),
         (5, 0, 2, -5.027447),
         (5, 1, 6, -3.655382),
         (4, 3, 14, -7.206499),
         (5, 2, 10, -1.386458),
         (6, 0, 2, -0.493528),
         (6, 1, 4, -0.217889)],
        {'Ecoul': 8200.970142,
         'Eenuc': -49169.103742,
         'Ekin': 20664.75611,
         'Etot': -20671.256539,
         'Exc': -367.879049}),
 'Pr': (59,
        [(1, 0, 2, -1457.338067),
         (2, 0, 2, -227.426363),
         (2, 1, 6, -215.418313),
         (3, 0, 2, -48.924994),
         (3, 1, 6, -43.692548),
         (3, 2, 10, -33.913996),
         (4, 0, 2, -9.577447),
         (4, 1, 6, -7.613108),
         (4, 2, 10, -4.154228),
         (5, 0, 2, -1.296106),
         (5, 1, 6, -0.778046),
         (4, 3, 3, -0.155138),
         (6, 0, 2, -0.124465)],
        {'Ecoul': 3508.213147,
         'Eenuc': -21136.965209,
         'Ekin': 8913.501128,
         'Etot': -8917.664369,
         'Exc': -202.413435}),
 'Pt': (78,
        [(1, 0, 2, -2613.096532),
         (2, 0, 2, -434.858003),
         (2, 1, 6, -417.96053),
         (3, 0, 2, -101.274869),
         (3, 1, 6, -93.309108),
         (3, 2, 10, -78.400271),
         (4, 0, 2, -21.110651),
         (4, 1, 6, -17.697297),
         (4, 2, 10, -11.419476),
         (5, 0, 2, -2.950526),
         (5, 1, 6, -1.884256),
         (4, 3, 14, -3.038049),
         (5, 2, 9, -0.273634),
         (6, 0, 1, -0.161308)],
        {'Ecoul': 6958.348706,
         'Eenuc': -41279.089979,
         'Ekin': 17320.592847,
         'Etot': -17326.576377,
         'Exc': -326.42795}),
 'Ra': (88,
        [(1, 0, 2, -3362.736563),
         (2, 0, 2, -577.101208),
         (2, 1, 6, -557.513214),
         (3, 0, 2, -142.632426),
         (3, 1, 6, -133.12325),
         (3, 2, 10, -115.306476),
         (4, 0, 2, -34.525628),
         (4, 1, 6, -30.221208),
         (4, 2, 10, -22.208125),
         (5, 0, 2, -7.139137),
         (5, 1, 6, -5.547203),
         (4, 3, 14, -11.181066),
         (5, 2, 10, -2.819853),
         (6, 0, 2, -1.05135),
         (6, 1, 6, -0.634674),
         (7, 0, 2, -0.113732)],
        {'Ecoul': 9045.587062,
         'Eenuc': -54820.47465,
         'Ekin': 23081.867528,
         'Etot': -23088.688083,
         'Exc': -395.668023}),
 'Rb': (37,
        [(1, 0, 2, -540.957115),
         (2, 0, 2, -71.291202),
         (2, 1, 6, -64.784678),
         (3, 0, 2, -10.513861),
         (3, 1, 6, -8.165416),
         (3, 2, 10, -3.915508),
         (4, 0, 2, -1.135051),
         (4, 1, 6, -0.59217),
         (5, 0, 1, -0.085375)],
        {'Ecoul': 1239.168864,
         'Eenuc': -7013.68955,
         'Ekin': 2933.940454,
         'Etot': -2936.337293,
         'Exc': -95.757061}),
 'Re': (75,
        [(1, 0, 2, -2407.665572),
         (2, 0, 2, -397.087707),
         (2, 1, 6, -380.982869),
         (3, 0, 2, -91.149193),
         (3, 1, 6, -83.634578),
         (3, 2, 10, -69.57676),
         (4, 0, 2, -18.454325),
         (4, 1, 6, -15.295495),
         (4, 2, 10, -9.516816),
         (5, 0, 2, -2.567348),
         (5, 1, 6, -1.631227),
         (4, 3, 14, -1.92508),
         (5, 2, 5, -0.258639),
         (6, 0, 2, -0.186859)],
        {'Ecoul': 6337.535878,
         'Eenuc': -37586.722924,
         'Ekin': 15774.543959,
         'Etot': -15780.236024,
         'Exc': -305.592936}),
 'Rh': (45,
        [(1, 0, 2, -821.136773),
         (2, 0, 2, -116.80695),
         (2, 1, 6, -108.357665),
         (3, 0, 2, -20.765603),
         (3, 1, 6, -17.415299),
         (3, 2, 10, -11.21725),
         (4, 0, 2, -2.825505),
         (4, 1, 6, -1.806456),
         (4, 2, 8, -0.239422),
         (5, 0, 1, -0.154624)],
        {'Ecoul': 1923.261557,
         'Eenuc': -11155.54306,
         'Ekin': 4680.264572,
         'Etot': -4683.301031,
         'Exc': -131.2841}),
 'Rn': (86,
        [(1, 0, 2, -3204.756288),
         (2, 0, 2, -546.57796),
         (2, 1, 6, -527.533025),
         (3, 0, 2, -133.369144),
         (3, 1, 6, -124.172862),
         (3, 2, 10, -106.945006),
         (4, 0, 2, -31.230804),
         (4, 1, 6, -27.108985),
         (4, 2, 10, -19.449994),
         (5, 0, 2, -5.889683),
         (5, 1, 6, -4.408702),
         (4, 3, 14, -8.953318),
         (5, 2, 10, -1.911329),
         (6, 0, 2, -0.62657),
         (6, 1, 6, -0.29318)],
        {'Ecoul': 8632.016039,
         'Eenuc': -51966.120382,
         'Ekin': 21854.6727,
         'Etot': -21861.346869,
         'Exc': -381.915225}),
 'Ru': (44,
        [(1, 0, 2, -782.918621),
         (2, 0, 2, -110.536054),
         (2, 1, 6, -102.333649),
         (3, 0, 2, -19.366692),
         (3, 1, 6, -16.145217),
         (3, 2, 10, -10.195668),
         (4, 0, 2, -2.628363),
         (4, 1, 6, -1.667549),
         (4, 2, 7, -0.210375),
         (5, 0, 1, -0.152834)],
        {'Ecoul': 1824.80092,
         'Eenuc': -10573.32364,
         'Ekin': 4436.031047,
         'Etot': -4438.981228,
         'Exc': -126.489556}),
 'S': (16,
       [(1, 0, 2, -87.789937),
        (2, 0, 2, -7.69994),
        (2, 1, 6, -5.751257),
        (3, 0, 2, -0.630912),
        (3, 1, 4, -0.261676)],
       {'Ecoul': 176.68795,
        'Eenuc': -945.141155,
        'Ekin': 395.878143,
        'Etot': -396.716081,
        'Exc': -24.141019}),
 'Sb': (51,
        [(1, 0, 2, -1070.823495),
         (2, 0, 2, -159.171745),
         (2, 1, 6, -149.214271),
         (3, 0, 2, -31.098242),
         (3, 1, 6, -26.956184),
         (3, 2, 10, -19.239895),
         (4, 0, 2, -5.04964),
         (4, 1, 6, -3.646579),
         (4, 2, 10, -1.297338),
         (5, 0, 2, -0.445605),
         (5, 1, 3, -0.185623)],
        {'Ecoul': 2544.726589,
         'Eenuc': -15001.364743,
         'Ekin': 6306.85786,
         'Etot': -6310.376268,
         'Exc': -160.595974}),
 'Sc': (21,
        [(1, 0, 2, -160.184109),
         (2, 0, 2, -17.206464),
         (2, 1, 6, -14.240006),
         (3, 0, 2, -1.988378),
         (3, 1, 6, -1.233165),
         (3, 2, 1, -0.13108),
         (4, 0, 2, -0.156478)],
        {'Ecoul': 318.555762,
         'Eenuc': -1797.853811,
         'Ekin': 757.5252,
         'Etot': -758.679275,
         'Exc': -36.906426}),
 'Se': (34,
        [(1, 0, 2, -451.300258),
         (2, 0, 2, -57.311948),
         (2, 1, 6, -51.514388),
         (3, 0, 2, -7.547186),
         (3, 1, 6, -5.553517),
         (3, 2, 10, -2.011392),
         (4, 0, 2, -0.621248),
         (4, 1, 4, -0.245806)],
        {'Ecoul': 1028.192719,
         'Eenuc': -5738.619928,
         'Ekin': 2395.934307,
         'Etot': -2398.11144,
         'Exc': -83.618538}),
 'Si': (14,
        [(1, 0, 2, -65.184426),
         (2, 0, 2, -5.075056),
         (2, 1, 6, -3.514938),
         (3, 0, 2, -0.398139),
         (3, 1, 2, -0.153293)],
        {'Ecoul': 131.767814,
         'Eenuc': -687.900675,
         'Ekin': 287.48774,
         'Etot': -288.198397,
         'Exc': -19.553276}),
 'Sm': (62,
        [(1, 0, 2, -1617.183426),
         (2, 0, 2, -255.498846),
         (2, 1, 6, -242.729726),
         (3, 0, 2, -55.731133),
         (3, 1, 6, -50.08426),
         (3, 2, 10, -39.528656),
         (4, 0, 2, -10.844667),
         (4, 1, 6, -8.672685),
         (4, 2, 10, -4.814978),
         (5, 0, 2, -1.408552),
         (5, 1, 6, -0.835987),
         (4, 3, 6, -0.21776),
         (6, 0, 2, -0.128259)],
        {'Ecoul': 3962.921783,
         'Eenuc': -23800.753315,
         'Ekin': 10026.812733,
         'Etot': -10031.25909,
         'Exc': -220.24029}),
 'Sn': (50,
        [(1, 0, 2, -1026.762169),
         (2, 0, 2, -151.523991),
         (2, 1, 6, -141.821093),
         (3, 0, 2, -29.125969),
         (3, 1, 6, -25.117913),
         (3, 2, 10, -17.657276),
         (4, 0, 2, -4.546335),
         (4, 1, 6, -3.211998),
         (4, 2, 10, -1.004952),
         (5, 0, 2, -0.369349),
         (5, 1, 2, -0.14445)],
        {'Ecoul': 2438.225987,
         'Eenuc': -14319.014475,
         'Ekin': 6016.512431,
         'Etot': -6019.953353,
         'Exc': -155.677296}),
 'Sr': (38,
        [(1, 0, 2, -572.870169),
         (2, 0, 2, -76.491823),
         (2, 1, 6, -69.745941),
         (3, 0, 2, -11.771585),
         (3, 1, 6, -9.301863),
         (3, 2, 10, -4.813498),
         (4, 0, 2, -1.455317),
         (4, 1, 6, -0.844489),
         (5, 0, 2, -0.131793)],
        {'Ecoul': 1308.954372,
         'Eenuc': -7465.641817,
         'Ekin': 3126.990706,
         'Etot': -3129.453161,
         'Exc': -99.756422}),
 'Ta': (73,
        [(1, 0, 2, -2275.371387),
         (2, 0, 2, -372.828724),
         (2, 1, 6, -357.248334),
         (3, 0, 2, -84.658467),
         (3, 1, 6, -77.440942),
         (3, 2, 10, -63.942521),
         (4, 0, 2, -16.713337),
         (4, 1, 6, -13.71981),
         (4, 2, 10, -8.265848),
         (5, 0, 2, -2.223807),
         (5, 1, 6, -1.37653),
         (4, 3, 14, -1.199347),
         (5, 2, 3, -0.182464),
         (6, 0, 2, -0.174814)],
        {'Ecoul': 5949.357668,
         'Eenuc': -35243.460062,
         'Ekin': 14790.376452,
         'Etot': -14795.884202,
         'Exc': -292.15826}),
 'Tb': (65,
        [(1, 0, 2, -1785.331942),
         (2, 0, 2, -285.121013),
         (2, 1, 6, -271.590585),
         (3, 0, 2, -62.851563),
         (3, 1, 6, -56.785113),
         (3, 2, 10, -45.443863),
         (4, 0, 2, -12.120486),
         (4, 1, 6, -9.735637),
         (4, 2, 10, -5.467662),
         (5, 0, 2, -1.513669),
         (5, 1, 6, -0.88723),
         (4, 3, 9, -0.256311),
         (6, 0, 2, -0.131677)],
        {'Ecoul': 4461.481521,
         'Eenuc': -26663.609136,
         'Ekin': 11218.204349,
         'Etot': -11222.941975,
         'Exc': -239.018708}),
 'Tc': (43,
        [(1, 0, 2, -745.742024),
         (2, 0, 2, -104.567508),
         (2, 1, 6, -96.61021),
         (3, 0, 2, -18.135303),
         (3, 1, 6, -15.041738),
         (3, 2, 10, -9.33986),
         (4, 0, 2, -2.550712),
         (4, 1, 6, -1.64323),
         (4, 2, 5, -0.270262),
         (5, 0, 2, -0.183636)],
        {'Ecoul': 1724.900741,
         'Eenuc': -10004.790618,
         'Ekin': 4199.331793,
         'Etot': -4202.188857,
         'Exc': -121.630774}),
 'Te': (52,
        [(1, 0, 2, -1115.831819),
         (2, 0, 2, -167.021776),
         (2, 1, 6, -156.808583),
         (3, 0, 2, -33.137485),
         (3, 1, 6, -28.860685),
         (3, 2, 10, -20.887801),
         (4, 0, 2, -5.572846),
         (4, 1, 6, -4.100084),
         (4, 2, 10, -1.608381),
         (5, 0, 2, -0.520997),
         (5, 1, 4, -0.226594)],
        {'Ecoul': 2653.969369,
         'Eenuc': -15702.061477,
         'Ekin': 6605.034216,
         'Etot': -6608.631413,
         'Exc': -165.573522}),
 'Th': (90,
        [(1, 0, 2, -3524.439052),
         (2, 0, 2, -608.350958),
         (2, 1, 6, -588.218112),
         (3, 0, 2, -152.079741),
         (3, 1, 6, -142.25581),
         (3, 2, 10, -123.846396),
         (4, 0, 2, -37.814094),
         (4, 1, 6, -33.325252),
         (4, 2, 10, -24.955184),
         (5, 0, 2, -8.287057),
         (5, 1, 6, -6.58281),
         (4, 3, 14, -13.397389),
         (5, 2, 10, -3.625729),
         (6, 0, 2, -1.333769),
         (6, 1, 6, -0.846921),
         (6, 2, 2, -0.172896),
         (7, 0, 2, -0.135872)],
        {'Ecoul': 9483.654939,
         'Eenuc': -57774.567583,
         'Ekin': 24346.845656,
         'Etot': -24353.832231,
         'Exc': -409.765243}),
 'Ti': (22,
        [(1, 0, 2, -177.276643),
         (2, 0, 2, -19.457901),
         (2, 1, 6, -16.285339),
         (3, 0, 2, -2.258007),
         (3, 1, 6, -1.422947),
         (3, 2, 2, -0.17001),
         (4, 0, 2, -0.167106)],
        {'Ecoul': 355.232355,
         'Eenuc': -2008.740849,
         'Ekin': 846.050533,
         'Etot': -847.277216,
         'Exc': -39.819256}),
 'Tl': (81,
        [(1, 0, 2, -2827.569408),
         (2, 0, 2, -474.953368),
         (2, 1, 6, -457.255971),
         (3, 0, 2, -112.52218),
         (3, 1, 6, -104.099296),
         (3, 2, 10, -88.328299),
         (4, 0, 2, -24.471512),
         (4, 1, 6, -20.797078),
         (4, 2, 10, -14.008848),
         (5, 0, 2, -3.811512),
         (5, 1, 6, -2.59873),
         (4, 3, 14, -4.835747),
         (5, 2, 10, -0.674544),
         (6, 0, 2, -0.28502),
         (6, 1, 1, -0.101507)],
        {'Ecoul': 7576.942724,
         'Eenuc': -45137.454222,
         'Ekin': 18950.710388,
         'Etot': -18956.957627,
         'Exc': -347.156518}),
 'Tm': (69,
        [(1, 0, 2, -2022.471608),
         (2, 0, 2, -327.05712),
         (2, 1, 6, -312.510608),
         (3, 0, 2, -72.873753),
         (3, 1, 6, -66.239338),
         (3, 2, 10, -53.835494),
         (4, 0, 2, -13.865665),
         (4, 1, 6, -11.187151),
         (4, 2, 10, -6.350307),
         (5, 0, 2, -1.64999),
         (5, 1, 6, -0.950748),
         (4, 3, 13, -0.28312),
         (6, 0, 2, -0.135953)],
        {'Ecoul': 5196.611124,
         'Eenuc': -30799.515253,
         'Ekin': 12931.649735,
         'Etot': -12936.786494,
         'Exc': -265.532099}),
 'U': (92,
       [(1, 0, 2, -3689.355141),
        (2, 0, 2, -639.778728),
        (2, 1, 6, -619.10855),
        (3, 0, 2, -161.118073),
        (3, 1, 6, -150.97898),
        (3, 2, 10, -131.977358),
        (4, 0, 2, -40.528084),
        (4, 1, 6, -35.853321),
        (4, 2, 10, -27.123212),
        (5, 0, 2, -8.824089),
        (5, 1, 6, -7.018092),
        (4, 3, 14, -15.02746),
        (5, 2, 10, -3.866175),
        (6, 0, 2, -1.325976),
        (6, 1, 6, -0.822538),
        (5, 3, 3, -0.366543),
        (6, 2, 1, -0.14319),
        (7, 0, 2, -0.130948)],
       {'Ecoul': 9991.594178,
        'Eenuc': -60876.210623,
        'Ekin': 25651.231184,
        'Etot': -25658.417889,
        'Exc': -425.032628}),
 'V': (23,
       [(1, 0, 2, -195.224014),
        (2, 0, 2, -21.815346),
        (2, 1, 6, -18.435189),
        (3, 0, 2, -2.526904),
        (3, 1, 6, -1.610516),
        (3, 2, 3, -0.204634),
        (4, 0, 2, -0.175968)],
       {'Ecoul': 395.162964,
        'Eenuc': -2234.347114,
        'Ekin': 940.377057,
        'Etot': -941.678904,
        'Exc': -42.871811}),
 'W': (74,
       [(1, 0, 2, -2341.042887),
        (2, 0, 2, -384.856157),
        (2, 1, 6, -369.013973),
        (3, 0, 2, -87.867792),
        (3, 1, 6, -80.502102),
        (3, 2, 10, -66.724787),
        (4, 0, 2, -17.570797),
        (4, 1, 6, -14.495102),
        (4, 2, 10, -8.879693),
        (5, 0, 2, -2.396018),
        (5, 1, 6, -1.504457),
        (4, 3, 14, -1.550835),
        (5, 2, 4, -0.220603),
        (6, 0, 2, -0.181413)],
       {'Ecoul': 6141.634533,
        'Eenuc': -36404.087502,
        'Ekin': 15277.849307,
        'Etot': -15283.448822,
        'Exc': -298.84516}),
 'Xe': (54,
        [(1, 0, 2, -1208.688993),
         (2, 0, 2, -183.327495),
         (2, 1, 6, -172.599583),
         (3, 0, 2, -37.415454),
         (3, 1, 6, -32.867042),
         (3, 2, 10, -24.37823),
         (4, 0, 2, -6.67834),
         (4, 1, 6, -5.063802),
         (4, 2, 10, -2.286666),
         (5, 0, 2, -0.672086),
         (5, 1, 6, -0.309835)],
        {'Ecoul': 2880.919348,
         'Eenuc': -17159.159426,
         'Ekin': 7225.097817,
         'Etot': -7228.856107,
         'Exc': -175.713845}),
 'Y': (39,
       [(1, 0, 2, -605.631981),
        (2, 0, 2, -81.789102),
        (2, 1, 6, -74.803201),
        (3, 0, 2, -12.992217),
        (3, 1, 6, -10.399926),
        (3, 2, 10, -5.671499),
        (4, 0, 2, -1.697124),
        (4, 1, 6, -1.02449),
        (4, 2, 1, -0.108691),
        (5, 0, 2, -0.150727)],
       {'Ecoul': 1384.603788,
        'Eenuc': -7937.187681,
        'Ekin': 3326.984254,
        'Etot': -3329.520604,
        'Exc': -103.920964}),
 'Yb': (70,
        [(1, 0, 2, -2084.069389),
         (2, 0, 2, -337.978976),
         (2, 1, 6, -323.178219),
         (3, 0, 2, -75.47663),
         (3, 1, 6, -68.698655),
         (3, 2, 10, -56.026315),
         (4, 0, 2, -14.312076),
         (4, 1, 6, -11.558246),
         (4, 2, 10, -6.574963),
         (5, 0, 2, -1.683886),
         (5, 1, 6, -0.966137),
         (4, 3, 14, -0.286408),
         (6, 0, 2, -0.136989)],
        {'Ecoul': 5393.365097,
         'Eenuc': -31891.798397,
         'Ekin': 13382.810298,
         'Etot': -13388.048594,
         'Exc': -272.425592}),
 'Zn': (30,
        [(1, 0, 2, -344.969756),
         (2, 0, 2, -41.531323),
         (2, 1, 6, -36.648765),
         (3, 0, 2, -4.573041),
         (3, 1, 6, -3.022363),
         (3, 2, 10, -0.398944),
         (4, 0, 2, -0.222725)],
        {'Ecoul': 774.056522,
         'Eenuc': -4257.166094,
         'Ekin': 1774.693645,
         'Etot': -1776.57385,
         'Exc': -68.157922}),
 'Zr': (40,
        [(1, 0, 2, -639.292236),
         (2, 0, 2, -87.237062),
         (2, 1, 6, -80.010043),
         (3, 0, 2, -14.230432),
         (3, 1, 6, -11.514415),
         (3, 2, 10, -6.544643),
         (4, 0, 2, -1.918971),
         (4, 1, 6, -1.186597),
         (4, 2, 2, -0.150673),
         (5, 0, 2, -0.162391)],
        {'Ecoul': 1464.236106,
         'Eenuc': -8426.900586,
         'Ekin': 3534.12421,
         'Etot': -3536.737751,
         'Exc': -108.19748})}

##########################################################################################
# Entry points for client code
##########################################################################################

_symb2Z = {k: v[0] for (k,v) in _neutral_entries.items()}

_Z2symb = {v: k for (k,v) in _symb2Z.items()}

allsymbols = [k for k in _symb2Z]


def Z_from_symbol(symbol):
    """Returns the atomic number from the element symbol."""
    return _symb2Z[symbol]


def symbol_from_Z(Z):
    """Returns the element symbol from atomic number."""
    return _Z2symb[Z]


class NistState(namedtuple("NistState", "n, l, occ, eig")):
    """Named tuple with info on the orbital"""


class NistEntry(namedtuple("NistEntry", "Z, states, energies")):
    """
    Database entry:

    A NistEntry has the atomic number Z, a list of NistState object and
    a dictionary with the energies in Ha units.
    """
    def __str__(self):
        lines = []
        app = lines.append

        app("Element: %s, Z: %d" % (self.symbol, self.Z))
        app("Quantum numbers, occupations, eigenvalues [Ha]:")
        for s in self.states:
            app("\t" + str(s).replace(s.__class__.__name__,""))

        app("Atomic energies [Ha]:")
        app("\t" + str(self.energies))

        return "\n".join(lines)

    @property
    def symbol(self):
        """Chemical symbol"""
        return symbol_from_Z(self.Z)

    def to_table(self):
        """Returns a table of strings whose columns contain ["n", "l", "occ", "eigval [Ha]"]]"""
        table = [["n", "l", "occ", "eigval [Ha]"]]
        for state in self.states:
            table.append([state.n, state.l, state.occ, state.eig])

        return table


def get_neutral_entry(symbol):
    """Retrieve an entry of the data from the element symbol or from Z (neutral atoms)."""
    if isinstance(symbol, int):
        symbol = symbol_from_Z(symbol)

    entry = _neutral_entries[symbol]
    states = [NistState(n=s[0], l=s[1], occ=s[2], eig=s[3]) for s in entry[1]]
    return NistEntry(Z=entry[0], states=states, energies=entry[2])


#def get_cation_entry(symbol):
#    "Retrieve an entry of the data from the element symbol (cations)"
#    entry = _cation_entries[symbol]
#    states = [NistState(n=s[0], l=s[1], occ=s[2], eig=s[3]) for s in entry[1]]
#    return NistEntry(Z=entry[0], states=states, energies=entry[2])
